 require "Window"

local EsperResource = {}

local wnd
local currentColor

-- screen size
local nScreenWidth,nScreenHeight = Apollo.GetScreenSize()

-- default settings
local tDefaultSettings = {
	tManaBar = {
		tPos = {
			l = (nScreenWidth / 2) - 130,
			t = (nScreenHeight / 2) + 250,
			r = (nScreenWidth / 2) + 130,
			b = ((nScreenHeight / 2) + 250) + 51,
		},
		strFont = "Nameplates",
		tTextFormat = {
			bBoth = true,
			bValue = false,
			bPercent = false,
			bHide = false,
		},
		strColor = "00ffff",
	},
	tPsiPoints = {
		tPos = {
			l = (nScreenWidth / 2) - 31.5,
			t = (nScreenHeight / 2) + 160,
			r = (nScreenWidth / 2) + 31.5,
			b = ((nScreenHeight / 2) + 160 )+ 73,
		},
		strFont = "CRB_HeaderGigantic_O",
		strColor = "00ffff",
	},
	bShowAlways = true,
	bMode = true, -- heal mode = true, dps = false
	bShowPsi = true,
}

-- local functions
local function Merge(t1, t2)
    for k, v in pairs(t2) do
        if (type(v) == "table") and (type(t1[k] or false) == "table") then
            Merge(t1[k], t2[k])
        else
            t1[k] = v
        end
    end
    return t1
end

local function FormatNumber(n)
	n = math.floor(string.format("%.0f", n))
	return n
end

-- initialize
function EsperResource:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    self.tConfig = {}
    self.tConfig.tManaBar = {}
    self.tConfig.tManaBar.tPos = {}
    self.tConfig.tPsiPoints = {}
    self.tConfig.tPsiPoints.tPos = {}

    return o
end

function EsperResource:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, tDependencies)
end

-- load
function EsperResource:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("EsperResource.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-- doc loaded
function EsperResource:OnDocLoaded()
	-- player check
	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end

-- instance
local EsperResourceInst = EsperResource:new()
EsperResourceInst:Init()

function EsperResource:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	-- save pos's again for insurance
	self:SavePos(self.wndManaBar,self.tConfig.tManaBar.tPos)
	self:SavePos(self.wndPsiPoints,self.tConfig.tPsiPoints.tPos)

	local tSave = self.tConfig
	return tSave
end

-- save
function EsperResource:OnRestore(eType,tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if tData then
		Merge(self.tConfig,tData)
	else
		Merge(self.tConfig,tDefaultSettings)
	end
end

-- restore
function EsperResource:OnCharacterCreated()
	-- if class not esper return
	if GameLib.GetPlayerUnit():GetClassId() == 3 then
		-- load forms
		self.wndManaBar = Apollo.LoadForm("EsperResource.xml", "ResourceBar", nil, self)
		self.wndPsiPoints = Apollo.LoadForm("EsperResource.xml", "PsiPoints", nil, self)
		self.wndOptions = Apollo.LoadForm("EsperResource.xml", "Options", nil, self)
		self.wndOptions:Show(false)

		-- reg events
		Apollo.RegisterEventHandler("VarChange_FrameCount", "UpdateFrames", self)
		Apollo.RegisterSlashCommand("er", "OnSlashCommand", self)

		self.bPsiFonts = false
		self.bManaFonts = false

		-- if no save data loaded
		if self.tConfig.bMode == nil then
			self:Reset()
			Print("[EsperResource]: Loaded.")
			Print("[EsperResource]: No saved data found, settings set to defaults. Opening options for configuration.")
			self.wndOptions:Show(true)
		else
			self:ApplyPositions()
			Print("[EsperResource]: Loaded. ")
			Print("[EsperResource]: Type '/er' for options.")
		end
		self:SetAllOptions()
	else 
		return
	end
end

-- update
function EsperResource:UpdateFrames()
	-- healer mode
	if self.tConfig.bMode then
		-- set values
		local nMana = GameLib.GetPlayerUnit():GetMana()
		local nMaxMana = GameLib.GetPlayerUnit():GetMaxMana()
		local strMana = self:FormatMana(nMana, nMaxMana)
		self.wndManaBar:FindChild("ManaBar"):SetMax(nMaxMana)
		self.wndManaBar:FindChild("ManaBar"):SetProgress(nMana)
		self.wndManaBar:FindChild("Focus"):SetText(strMana)
	end

	-- psi points
	local nPsiCurr = GameLib.GetPlayerUnit():GetResource(1)
	local nPsiMax = GameLib.GetPlayerUnit():GetMaxResource(1)
	self.wndPsiPoints:SetText(nPsiCurr)

	-- default show/hide if locked
	if self.tConfig.bFramesLocked then
		self:Visibility()
	end
end

-- show/hide
function EsperResource:Visibility()
	if self.tConfig.bShowAlways then
		if self.tConfig.bMode then
			self.wndManaBar:Show(true)
		else
			self.wndManaBar:Show(false)
		end
		if self.tConfig.bShowPsi then
			self.wndPsiPoints:Show(true)
		else
			self.wndPsiPoints:Show(false)
		end
	else
		if GameLib.GetPlayerUnit():IsInCombat() then
			if self.tConfig.bMode then
				self.wndManaBar:Show(true)
			else
				self.wndManaBar:Show(false)
			end
			if self.tConfig.bShowPsi then
				self.wndPsiPoints:Show(true)
			else
				self.wndPsiPoints:Show(false)
			end
		else
			self.wndManaBar:Show(false)
			self.wndPsiPoints:Show(false)
		end
	end
end

-- slash cmds
function EsperResource:OnSlashCommand()
	self.wndOptions:Show(true)
end

-- frame resets
function EsperResource:Reset()
	self.tConfig = nil
	self.tConfig = {}
	Merge(self.tConfig, tDefaultSettings)
	self:ApplyPositions()
end

function EsperResource:ApplyPositions()
	local x = self.tConfig.tManaBar.tPos
	local y = self.tConfig.tPsiPoints.tPos

	self.wndManaBar:SetAnchorOffsets(x.l,x.t,x.r,x.b)
	self.wndPsiPoints:SetAnchorOffsets(y.l,y.t,y.r,y.b)
end

function EsperResource:SavePos(wnd,tbl)
	local x = tbl
	x.l,x.t,x.r,x.b = wnd:GetAnchorOffsets()
end

-- frame (un)lock
function EsperResource:Lock(bool)
	self.tConfig.bFramesLocked = bool
	if not self.tConfig.bFramesLocked then
		-- mana bar
		self.wndManaBar:SetStyle("Moveable", true)
		self.wndManaBar:SetStyle("IngoreMouse", false)
		self.wndManaBar:SetStyle("Picture", true)
		-- self.wndManaBar:FindChild("BG"):SetStyle("Picture", true)
		self.wndManaBar:FindChild("MoveMeText"):Show(true)

		-- psi points
		self.wndPsiPoints:SetStyle("Moveable", true)
		self.wndPsiPoints:SetStyle("IngoreMouse", false)
		self.wndPsiPoints:SetStyle("Picture", true)
		self.wndPsiPoints:FindChild("MoveMeText"):Show(true)
	else
		-- mana bar
		self.wndManaBar:SetStyle("Moveable", false)
		self.wndManaBar:SetStyle("IngoreMouse", true)
		self.wndManaBar:SetStyle("Picture", false)
		-- self.wndManaBar:FindChild("BG"):SetStyle("Picture", false)
		self.wndManaBar:FindChild("MoveMeText"):Show(false)

		-- psi points
		self.wndPsiPoints:SetStyle("Moveable", false)
		self.wndPsiPoints:SetStyle("IngoreMouse", true)
		self.wndPsiPoints:SetStyle("Picture", false)
		self.wndPsiPoints:FindChild("MoveMeText"):Show(false)

		-- save
		self:SavePos(self.wndManaBar,self.tConfig.tManaBar.tPos)
		self:SavePos(self.wndPsiPoints,self.tConfig.tPsiPoints.tPos)
	end
end

function EsperResource:ToggleOptions( wndHandler, wndControl, eMouseButton )
	self.wndOptions:Show(false)
end

function EsperResource:InitializeReset( wndHandler, wndControl, eMouseButton )
	self.wndOptions:FindChild("ResetWindow"):Show(true)
end

function EsperResource:ConfirmReset( wndHandler, wndControl, eMouseButton )
	self:Reset()
	RequestReloadUI()
end

function EsperResource:CancelReset( wndHandler, wndControl, eMouseButton )
	self.wndOptions:FindChild("ResetWindow"):Show(false)
end

function EsperResource:VisibleAlways( wndHandler, wndControl, eMouseButton )
	local bIsChecked = wndControl:IsChecked()
	self.tConfig.bShowAlways = bIsChecked
end

function EsperResource:VisibleCombat( wndHandler, wndControl, eMouseButton )
	local bIsChecked = not wndControl:IsChecked()
	self.tConfig.bShowAlways = bIsChecked
end

function EsperResource:FrameLock( wndHandler, wndControl, eMouseButton )
	self:Lock(true)
end

function EsperResource:FrameUnlock( wndHandler, wndControl, eMouseButton )
	self:Lock(false)
end

function EsperResource:ModeHealing( wndHandler, wndControl, eMouseButton )
	local bIsChecked = wndControl:IsChecked()
	self.tConfig.bMode = bIsChecked
end

function EsperResource:ModeDPS( wndHandler, wndControl, eMouseButton )
	local bIsChecked = not wndControl:IsChecked()
	self.tConfig.bMode = bIsChecked
end

function EsperResource:TogglePsiPoints( wndHandler, wndControl, eMouseButton )
	local bIsChecked = wndControl:IsChecked()
	if bIsChecked then
		self.tConfig.bShowPsi = true
	else
		self.tConfig.bShowPsi = false
	end
end

function EsperResource:TogglePsiPointFonts( wndHandler, wndControl, eMouseButton )
	if self.bManaFonts then
		self:ToggleManaBarFonts()
	end
	self.bPsiFonts = not self.bPsiFonts
	self.wndOptions:FindChild("PsiPointFonts"):Show(self.bPsiFonts)
end

function EsperResource:ToggleManaBarFonts( wndHandler, wndControl, eMouseButton )
	if self.bPsiFonts then
		self:TogglePsiPointFonts()
	end
	self.bManaFonts = not self.bManaFonts
	self.wndOptions:FindChild("ManaBarFonts"):Show(self.bManaFonts)
end

function EsperResource:SetManaFormatBoth( wndHandler, wndControl, eMouseButton )
	self.tConfig.tManaBar.tTextFormat.bBoth = true
	self.tConfig.tManaBar.tTextFormat.bValue = false
	self.tConfig.tManaBar.tTextFormat.bPercent = false
	self.tConfig.tManaBar.tTextFormat.bHide = false
end

function EsperResource:SetManaFormatValue( wndHandler, wndControl, eMouseButton )
	self.tConfig.tManaBar.tTextFormat.bBoth = false
	self.tConfig.tManaBar.tTextFormat.bValue = true
	self.tConfig.tManaBar.tTextFormat.bPercent = false
	self.tConfig.tManaBar.tTextFormat.bHide = false
end

function EsperResource:SetManaFormatPercent( wndHandler, wndControl, eMouseButton )
	self.tConfig.tManaBar.tTextFormat.bBoth = false
	self.tConfig.tManaBar.tTextFormat.bValue = false
	self.tConfig.tManaBar.tTextFormat.bPercent = true
	self.tConfig.tManaBar.tTextFormat.bHide = false
end

function EsperResource:SetManaFormatHide( wndHandler, wndControl, eMouseButton )
	self.tConfig.tManaBar.tTextFormat.bBoth = false
	self.tConfig.tManaBar.tTextFormat.bValue = false
	self.tConfig.tManaBar.tTextFormat.bPercent = false
	self.tConfig.tManaBar.tTextFormat.bHide = true
end

function EsperResource:SetPsiFontSubtitle( wndHandler, wndControl, eMouseButton )
	self.wndPsiPoints:SetFont("Subtitle")
	self.tConfig.tPsiPoints.strFont = "Subtitle"
	self.wndOptions:FindChild("PsiPointFontBtn"):SetText(self.tConfig.tPsiPoints.strFont)
end

function EsperResource:SetPsiFontHeaderGigantic( wndHandler, wndControl, eMouseButton )
	self.wndPsiPoints:SetFont("CRB_HeaderGigantic_O")
	self.tConfig.tPsiPoints.strFont = "CRB_HeaderGigantic_O"
	self.wndOptions:FindChild("PsiPointFontBtn"):SetText(self.tConfig.tPsiPoints.strFont)
end

function EsperResource:SetManaFontSubtitle( wndHandler, wndControl, eMouseButton )
	self.wndManaBar:FindChild("Focus"):SetFont("Subtitle")
	self.tConfig.tManaBar.strFont = "Subtitle"
	self.wndOptions:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
end

function EsperResource:SetManaFontNameplates( wndHandler, wndControl, eMouseButton )
	self.wndManaBar:FindChild("Focus"):SetFont("Nameplates")
	self.tConfig.tManaBar.strFont = "Nameplates"
	self.wndOptions:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
end

function EsperResource:SetManaFontCourier( wndHandler, wndControl, eMouseButton )
	self.wndManaBar:FindChild("Focus"):SetFont("Courier")
	self.tConfig.tManaBar.strFont = "Courier"
	self.wndOptions:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
end

function EsperResource:SetManaFontThick( wndHandler, wndControl, eMouseButton )
	self.wndManaBar:FindChild("Focus"):SetFont("Thick")
	self.tConfig.tManaBar.strFont = "Thick"
	self.wndOptions:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
end

function EsperResource:SetManaFontHeaderGigantic( wndHandler, wndControl, eMouseButton )
	self.wndManaBar:FindChild("Focus"):SetFont("CRB_HeaderGigantic_O")
	self.tConfig.tManaBar.strFont = "CRB_HeaderGigantic_O"
	self.wndOptions:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
end

function EsperResource:OnSetColors()
	local psiColor = self.wndOptions:FindChild("PsiPointsColor"):FindChild("EditBox"):GetText()
	local manaColor = self.wndOptions:FindChild("ManaBarColor"):FindChild("EditBox"):GetText()
	if string.len(psiColor) == 6 then 
		self.wndPsiPoints:SetTextColor("ff"..psiColor)
		self.tConfig.tPsiPoints.strColor = psiColor
	else 
		Print("[EsperResource]: Error psi point color not hex.") 
	end
	if string.len(manaColor) == 6 then
		self.wndManaBar:FindChild("ManaBar"):SetBarColor("ff"..manaColor)
		self.tConfig.tManaBar.strColor = manaColor
	else
		Print("[EsperResource]: Error mana color not hex.")
	end 
end

function EsperResource:FormatMana(nMana,nMaxMana)
	local strManaSmart = string.format("%s | %s", FormatNumber(nMana), (math.floor((nMana / nMaxMana)*100))).."%"
	local nManaValue = nMana
	local nManaPerc = math.floor((nMana / nMaxMana)*100).."%"
	if self.tConfig.tManaBar.tTextFormat.bBoth then
		return strManaSmart
	elseif self.tConfig.tManaBar.tTextFormat.bValue then
		return nManaValue
	elseif self.tConfig.tManaBar.tTextFormat.bPercent then
		return nManaPerc
	else
		return ""
	end
end

function EsperResource:SetAllOptions()
	local wnd = self.wndOptions
	-- visiblity
	wnd:FindChild("AlwaysBtn"):SetCheck(self.tConfig.bShowAlways)
	wnd:FindChild("CombatBtn"):SetCheck(not self.tConfig.bShowAlways)
	-- lock state
	wnd:FindChild("FrameLockBtn"):SetCheck(self.tConfig.bFramesLocked)
	wnd:FindChild("FrameUnlockBtn"):SetCheck(not self.tConfig.bFramesLocked)
	if self.tConfig.bFramesLocked then self:Lock(true) end
	-- mode
	wnd:FindChild("HealingBtn"):SetCheck(self.tConfig.bMode)
	wnd:FindChild("DPSBtn"):SetCheck(not self.tConfig.bMode)
	-- text format
	wnd:FindChild("BothBtn"):SetCheck(self.tConfig.tManaBar.tTextFormat.bBoth)
	wnd:FindChild("ValueBtn"):SetCheck(self.tConfig.tManaBar.tTextFormat.bValue)
	wnd:FindChild("PercentBtn"):SetCheck(self.tConfig.tManaBar.tTextFormat.bPercent)
	wnd:FindChild("NoDisplayBtn"):SetCheck(self.tConfig.tManaBar.tTextFormat.bHide)
	-- psi visible
	wnd:FindChild("PsiVisibleBtn"):SetCheck(self.tConfig.bShowPsi)
	-- fonts
	wnd:FindChild("PsiPointFontBtn"):SetText(self.tConfig.tPsiPoints.strFont)
	wnd:FindChild("ManaBarFontBtn"):SetText(self.tConfig.tManaBar.strFont)
	self.wndManaBar:FindChild("Focus"):SetFont(self.tConfig.tManaBar.strFont)
	self.wndPsiPoints:SetFont(self.tConfig.tPsiPoints.strFont)
	-- colors 
	wnd:FindChild("ManaBarColor"):FindChild("EditBox"):SetText(self.tConfig.tManaBar.strColor)
	wnd:FindChild("PsiPointsColor"):FindChild("EditBox"):SetText(self.tConfig.tPsiPoints.strColor)
	self.wndPsiPoints:SetTextColor("ff"..self.tConfig.tPsiPoints.strColor)
	self.wndManaBar:FindChild("ManaBar"):SetBarColor("ff"..self.tConfig.tManaBar.strColor)
end
